"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Card = exports.Summary = exports.Leaf = exports.Nary = exports.Binary = exports.Node = exports.DepthFirst = exports.Tree = void 0;
var util_1 = require("./util");
var Tree = (function () {
    function Tree() {
        this.jsonStr = '';
        this.root = null;
        this.history = [];
        this.summaryElement = new Summary();
    }
    Tree.create = function (url, result, error) {
        var tree = new Tree();
        var promise = tree.loadJson(url);
        promise.then(function (p) {
            if (error) {
                error.innerHTML = p;
                error.style.display = 'hide';
            }
            tree.toHTML(result);
        }).catch(function (p) {
            if (!error) {
                throw error;
            }
            error.innerHTML = p;
            error.style.display = 'block';
            error.style.visibility = 'visible';
        });
    };
    Tree.prototype.loadJson = function (url) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var httpRequest = new XMLHttpRequest();
            httpRequest.onreadystatechange = function () {
                if (httpRequest.readyState === 4) {
                    if (httpRequest.status === 200) {
                        this._loadJson(httpRequest.responseText, resolve, reject);
                    }
                    else {
                        reject(httpRequest.statusText);
                    }
                }
            }.bind(_this);
            httpRequest.onerror = function () {
                reject(httpRequest.statusText);
            }.bind(_this);
            httpRequest.open('GET', url, true);
            httpRequest.send();
        });
    };
    Tree.prototype._loadJson = function (json, resolve, reject) {
        try {
            this.jsonStr = json;
            this.fromJson();
            resolve('JSON loaded');
        }
        catch (err) {
            reject('JSON Parser Error!');
        }
    };
    Tree.prototype.fromJson = function () {
        var _this = this;
        this.root = Node.fromJson(JSON.parse(this.jsonStr));
        var dfs = new DepthFirst(this, function (x) { return x; });
        dfs.result.forEach(function (n) { return n.tree = _this; });
        this.summaryElement.parent = this.root;
        this.summaryElement.tree = this;
    };
    Tree.prototype.toHTML = function (node) {
        this.rootElement = node;
        var dfs = new DepthFirst(this, function (x) { return x; });
        dfs.result.forEach(function (n) {
            node.appendChild(n.toHtml());
            n.hide();
        });
        node.appendChild(this.summaryElement.toHtml());
        this.summaryElement.hide();
        this.root.show();
    };
    Tree.prototype.summary = function (node) {
        node.hide();
        this.summaryElement.parent = node;
        this.summaryElement.show();
    };
    return Tree;
}());
exports.Tree = Tree;
var DepthFirst = (function () {
    function DepthFirst(tree, func) {
        this.func = func;
        this.result = [];
        this.visit(tree.root);
    }
    DepthFirst.prototype.visit = function (node) {
        var _this = this;
        this.result.push(this.func(node));
        node.children.forEach(function (x) { return _this.visit(x); });
    };
    return DepthFirst;
}());
exports.DepthFirst = DepthFirst;
var Node = (function () {
    function Node(content, title, value, labels) {
        var e_1, _a;
        if (title === void 0) { title = ''; }
        if (value === void 0) { value = 0; }
        if (labels === void 0) { labels = []; }
        this.content = content;
        this.title = title;
        this.value = value;
        this.kind = '';
        this.nextName = 'Next';
        this.previousName = 'Back';
        this.labels = new Map();
        this.children = new Map();
        this.parent = null;
        this.variable = 'dt_variable_' + Node.counter++;
        this.radioButtons = [];
        this.card = new Card();
        try {
            for (var labels_1 = __values(labels), labels_1_1 = labels_1.next(); !labels_1_1.done; labels_1_1 = labels_1.next()) {
                var _b = labels_1_1.value, text = _b.text, value_1 = _b.value;
                this.labels.set(value_1, text);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (labels_1_1 && !labels_1_1.done && (_a = labels_1.return)) _a.call(labels_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    Node.parseChildren = function (node, json) {
        var e_2, _a;
        if (!json || !json.length) {
            return;
        }
        try {
            for (var json_1 = __values(json), json_1_1 = json_1.next(); !json_1_1.done; json_1_1 = json_1.next()) {
                var child = json_1_1.value;
                var childNode = Node.fromJson(child);
                node.children.set(childNode.value, childNode);
                childNode.parent = node;
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (json_1_1 && !json_1_1.done && (_a = json_1.return)) _a.call(json_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    Node.fromJson = function (json) {
        var kind = json.type || 'leaf';
        var node = null;
        switch (kind) {
            case 'leaf':
                node = new Leaf(json.content, json.title, json.value, json.action);
                break;
            case 'nary':
                node = new Nary(json.content, json.title, json.value, json.labels || []);
                Node.parseChildren(node, json.children);
                break;
            case 'binary':
                node = new Binary(json.content, json.title, json.value);
                Node.parseChildren(node, json.children);
                break;
            default:
                throw Error('Unknown node type.');
        }
        return node;
    };
    Node.prototype.toHtml = function () {
        util_1.default.addClass(this.card.div, this.kind);
        this.card.title.innerHTML = this.title;
        var content = document.createElement('span');
        util_1.default.addClass(content, 'CONTENT');
        content.innerHTML = this.content;
        this.card.content.appendChild(content);
        this.radios();
        this.buttons();
        return this.card.div;
    };
    Node.prototype.radios = function () {
        var e_3, _a;
        try {
            for (var _b = __values(this.labels), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 2), key = _d[0], value = _d[1];
                var content = util_1.default.makeNode('div', 'RADIOBUTTON');
                var radio = document.createElement('input');
                util_1.default.addClass(radio, 'RADIO');
                var radioId = 'dt_id_' + Node.labelCounter++;
                var labelId = 'dt_id_' + Node.labelCounter++;
                radio.type = 'radio';
                radio.id = radioId;
                radio.name = this.variable;
                radio.value = key.toString();
                this.radioButtons.push(radio);
                var label = document.createElement('label');
                util_1.default.addClass(label, 'LABEL');
                label.id = labelId;
                label.setAttribute('for', radioId);
                label.innerHTML = value;
                content.appendChild(radio);
                content.appendChild(label);
                this.card.content.appendChild(content);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    Node.prototype.buttons = function () {
        if (this.parent) {
            util_1.default.makeButton(this.previousName, this.firePrevious.bind(this), this.card.buttons, 'PREVIOUS');
        }
        util_1.default.makeButton(this.nextName, this.fireNext.bind(this), this.card.buttons, 'NEXT');
    };
    Node.prototype.fireNext = function () {
        var e_4, _a;
        try {
            for (var _b = __values(this.radioButtons), _c = _b.next(); !_c.done; _c = _b.next()) {
                var radio = _c.value;
                if (radio.checked) {
                    var value = radio.value;
                    var num = parseInt(value, 10);
                    this.tree.history.push(num);
                    this.hide();
                    this.children.get(num).show();
                    return;
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_4) throw e_4.error; }
        }
    };
    Node.prototype.firePrevious = function () {
        if (this.parent) {
            this.hide();
            this.parent.show();
        }
    };
    Node.prototype.show = function () {
        this.card.show();
    };
    Node.prototype.hide = function () {
        this.card.hide();
    };
    Node.counter = 0;
    Node.labelCounter = 0;
    return Node;
}());
exports.Node = Node;
var Binary = (function (_super) {
    __extends(Binary, _super);
    function Binary(content, title, value) {
        if (value === void 0) { value = 0; }
        var _this = _super.call(this, content, title, value, [{ text: 'Yes', value: 1 }, { text: 'No', value: 0 }]) || this;
        _this.content = content;
        _this.title = title;
        _this.value = value;
        _this.kind = 'binary';
        return _this;
    }
    return Binary;
}(Node));
exports.Binary = Binary;
var Nary = (function (_super) {
    __extends(Nary, _super);
    function Nary() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.kind = 'nary';
        return _this;
    }
    return Nary;
}(Node));
exports.Nary = Nary;
var Leaf = (function (_super) {
    __extends(Leaf, _super);
    function Leaf(content, title, value, action) {
        if (action === void 0) { action = ''; }
        var _this = _super.call(this, content, title, value) || this;
        _this.action = action;
        _this.kind = 'leaf';
        _this.nextName = 'Restart';
        return _this;
    }
    Leaf.prototype.buttons = function () {
        _super.prototype.buttons.call(this);
        this.summaryButton = util_1.default.makeButton('Summary', this.fireSummary.bind(this), this.card.buttons);
        if (this.action) {
            this.actionButton = util_1.default.makeButton('Go', this.fireAction.bind(this), this.card.buttons, 'ACTION');
            document.createElement('button');
        }
    };
    Leaf.prototype.fireAction = function () {
        window.open(this.action, '_blank');
    };
    Leaf.prototype.fireSummary = function () {
        this.tree.summary(this);
    };
    Leaf.prototype.fireNext = function () {
        this.hide();
        this.tree.root.show();
        this.tree.history = [];
    };
    return Leaf;
}(Node));
exports.Leaf = Leaf;
var Summary = (function (_super) {
    __extends(Summary, _super);
    function Summary() {
        var _this = _super.call(this, '', 'Summary') || this;
        _this.kind = 'summary';
        _this.nextName = 'Restart';
        _this.summaryElement = util_1.default.makeNode('div', 'SUMMARY');
        _this.card.content.appendChild(_this.summaryElement);
        return _this;
    }
    Summary.prototype.fireNext = function () {
        this.hide();
        this.tree.root.show();
        this.tree.history = [];
    };
    Summary.prototype.show = function () {
        _super.prototype.show.call(this);
        this.compileSummary();
    };
    Summary.prototype.compileSummary = function () {
        var _this = this;
        var count = 0;
        var hl = this.tree.history.length;
        var node = this.parent;
        var result = [];
        this.summaryElement.innerHTML = '';
        do {
            var answer = count ? node.labels.get(this.tree.history[hl - count]) : '';
            result.unshift(this.makeSummaryLine(node.content, answer));
            node = node.parent;
            count++;
        } while (node);
        result.forEach(function (x) { return _this.summaryElement.appendChild(x); });
    };
    Summary.prototype.makeSummaryLine = function (question, answer) {
        var div = util_1.default.makeNode('div', 'SUMMARYLINE');
        util_1.default.makeSpan(div, question, 'QUESTION');
        util_1.default.makeSpan(div, answer, 'ANSWER');
        return div;
    };
    return Summary;
}(Node));
exports.Summary = Summary;
var Card = (function () {
    function Card() {
        this.div = util_1.default.makeNode('div', 'NODE');
        this.title = util_1.default.makeNode('h1', 'TITLE');
        this.content = util_1.default.makeNode('div', 'CONTENT');
        this.buttons = util_1.default.makeNode('div', 'BUTTONS');
        this.div.appendChild(this.title);
        this.title.setAttribute('tabindex', '-1');
        this.div.appendChild(this.content);
        this.content.setAttribute('tabindex', '-1');
        this.div.appendChild(this.buttons);
    }
    Card.prototype.show = function () {
        this.div.style.display = 'block';
        this.title.setAttribute('aria-live', 'polite');
        this.content.setAttribute('tabindex', '0');
        this.title.setAttribute('tabindex', '0');
        this.title.focus();
    };
    Card.prototype.hide = function () {
        this.div.style.display = 'none';
        this.title.removeAttribute('aria-live');
        this.title.setAttribute('tabindex', '-1');
        this.content.setAttribute('tabindex', '-1');
    };
    return Card;
}());
exports.Card = Card;
//# sourceMappingURL=tree.js.map